/// The definition of a standard directive.
///
/// Note that this type is an input to the parser, not an output.
pub struct Directive {
    /// The provider's name for the directive.
    name: String,
    /// Description of the directive.
    doc: String,
    /// Fixed prefix term.
    prefix: String,
    /// The type of the prefix argument, if there is one.
    prefix_arg: Option<Param>,
    /// Fixed infix term.
    /// Can only be present if the prefix argument exists.
    infix: Option<String>,
    /// The type of the infix argument.
    /// Can only be present if the infix term exists.
    infix_arg: Option<Param>,
    /// Fixed suffix term.
    /// Can only be present if the infix argument exists.
    /// Note that this implies directives which have two terms have a prefix
    /// and an infix, not a prefix and a suffix.
    suffix: Option<String>,
    /// The type of the suffix argument.
    /// Can only be present if the suffix exists.
    suffix_arg: Option<Param>,
    /// The type of the elements of the list argument, if supported.
    list_arg: Option<Param>,
    // TODO(zeph): macro expansion?
}

/// Directive parameter.
pub struct Param {
    name: String,
    doc: String,
    typ: Type,
}

/// Types expected by directive arguments.
pub enum Type {
    /// Arbitrary text.
    Text,
    /// Speech text, possibly containing emphasis, shout, or whisper markers.
    Speech,
    /// Character in the current scene.
    Character,
    /// Duration.
    Duration,
    /// Asset requiring particular tags.
    Asset { tags: Vec<String> },
    /// Enumeration of literal options.
    Enum(Vec<String>),
}

#[cfg(test)]
mod tests {
    use super::*;
    use rstest::rstest;
}
